;*******************************************
; Written by Yuqiang Zhang, UNC, 2014
;
; Concepts illustrated:
;   - Plotting CMAQ data; also works for data using LCC project, like WRF and other RCM
;   - Drawing filled contours over a Lambert Conformal map
;   - Drawing U.S. states
;********************************************************************
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_code.ncl"   
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_csm.ncl"  
load "$NCARG_ROOT/lib/ncarg/nclscripts/wrf/WRF_contributed.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/contributed.ncl"
;load ".hluresfile"
 
begin

   ;read the MCIP file and the 2D lon&lat
   lonlatfile = "GRIDCRO2D_cesm36k_19971231"
   flonlat = addfile(lonlatfile,"r")
   lon2d_temp = flonlat->LON
   lat2d_temp = flonlat->LAT
   printVarSummary(lon2d_temp)
   lon2d = lon2d_temp(0,0,:,:)
   lat2d = lat2d_temp(0,0,:,:)
   printVarSummary(lon2d)
   printVarSummary(lat2d)
   lat1d = ndtooned(lat2d)
   lon1d = ndtooned(lon2d)
   nlatlon = dimsizes(lat1d)

;Whole Domain
;lat_min = min(ndtooned(lat2d))
;lat_max = max(ndtooned(lat2d))
;lon_min = min(ndtooned(lon2d))
;lon_max = max(ndtooned(lon2d))

;Subset the data/map for around Chesapeake Bay Region
lat_min = 33.
lat_max = 46.
lon_min = -88.
lon_max = -69.

;read in USGS Atlas Shapefile of U.S. States

fAin_shp = "Chesbay_N_Airshed.shp"
fA_shp   = addfile(fAin_shp,"r")
; Read data off shapefile
;
  geomDims      = getfilevardimsizes(fA_shp,"geometry")
  numFeatures   = geomDims(0)
  segments      = fA_shp->segments    ;U.S. segments
  geometry      = fA_shp->geometry    ;U.S. geometry
;  state_names   = fA_shp->STATE      ;U.S. state names
;  FIPS_names    = fA_shp->STATE_FIPS  ;U.S. state FIPS
  lat           = fA_shp->y           ;U.S. latitudes
  lon           = fA_shp->x           ;U.S. longitudes
  segsDims = dimsizes(segments)
  geomDims = dimsizes(geometry)
  latDims  = dimsizes(lat)
;
; Read global attributes

 geom_segIndex = fA_shp@geom_segIndex
  geom_numSegs  = fA_shp@geom_numSegs
  segs_xyzIndex = fA_shp@segs_xyzIndex
  segs_numPnts  = fA_shp@segs_numPnts
  layer_names   = fA_shp@layer_name


ij = getind_latlon2d(lat2d,lon2d,(/lat_min,lat_max/),(/lon_min,lon_max/))
;---Store to local variables for better code readability
  ilat1 = ij(0,0)
  ilat2 = ij(1,0)
  ilon1 = ij(0,1)
  ilon2 = ij(1,1)

;print(ilat1)
;print(ilat2)
;print(ilon1)
;print(ilon2)

;get the dimensions
;   dims=dimsizes(lon2d(ilat1:ilat2,ilon1:ilon2))
;   print(dims)
;   ndims = dimsizes(dims)
;   nlon=dims(ndims-1)                                ; assign # lat/lon points
;   nlat=dims(ndims-2)
;   print(nlon)
;   print(nlat)



;------------This Section uses shapefile data to get the average of a data array over all points that fall within each(selected) state(s) in the U.S..

;Collecting all indices pertaining to selected U.S. States
State_ind = ind(layer_names.eq."Chesbay_N_Airshed")

;State_ind = ind(state_names.eq."Delaware")

;Make new 2d lat/long indice arrays to hold all new indices for lat/lons in selected U.S states
lat1d_new      = new((/dimsizes(State_ind),dimsizes(lat1d)/),float)
lon1d_new      = new((/dimsizes(State_ind),dimsizes(lon1d)/),float)
;print(dimsizes(lat2d_new_ind))

State_num = dimsizes(State_ind)

;print(State_num)
;print(State_ind)


;---Create array to hold new data mask, and set all values to 0 initially.

data_mask_1d = new(nlatlon,integer)
  data_mask_1d = 0
 skip_check   = new(nlatlon,integer)
  skip_check   = 0


do i=0, State_num-1
    print("--------------------------------------------------")
    print("Inspecting U.S. state '" + layer_names(State_ind(i)) + "'...")
data_mask_1d = 0    ; Be sure to reset to 0 for every state

;---Some States have multiple segments
    startSegment = geometry(State_ind(i), geom_segIndex)
    numSegments  = geometry(State_ind(i), geom_numSegs)
print("Some States have multiple segments: Segments = " + numSegments)


    do seg=startSegment, startSegment+numSegments-1
      startPT = segments(seg, segs_xyzIndex)
      endPT   = startPT + segments(seg, segs_numPnts) - 1

;---Loop through each point on data grid and see if it's in this state.
      do n=0,nlatlon-1
;        nn = ii_latlon(n)     ; Get index of point we're checking

      ; This "if" statement speeds up code, by making sure we don't
; needlessly check a lat/lon point if:
;
; - we've already found it in another state
; - it doesn't fall within the general lat/lon box that covers this state
;
        if(skip_check(n).or.\
           lat1d(n).lt.min(lat(startPT:endPT)).or.\
           lat1d(n).gt.max(lat(startPT:endPT)).or.\
           lon1d(n).lt.min(lon(startPT:endPT)).or.\
           lon1d(n).gt.max(lon(startPT:endPT)))
           continue
        end if


;---Here's the check if the latitude point is in the state.
        if(gc_inout(lat1d(n),lon1d(n),\
                    lat(startPT:endPT),lon(startPT:endPT))) then
          data_mask_1d(n) = 1    ; This point is inside this state
          skip_check(n)   = 1    ; Don't check this point state
;         print(lat1d(n))
;         print(lon1d(n))
        end if


      end do
    end do    ; End of collecting points for this state

;---Count number of points found in this state
;print (data_mask_1d)
    ndm = num(data_mask_1d.eq.1)
    print(ndm + " data values found in Chesbay.")

; If there are values found in this state, then take new lat/lon indices to use for the PDF calculation below.
;

;---First Make sure "lat1d" has a missing value.
  if(.not.isatt(lat1d,"_FillValue")) then
    lat1d@_FillValue = default_fillvalue(typeof(lat1d))
  end if

;---First Make sure "lon1d" has a missing value.
  if(.not.isatt(lon1d,"_FillValue")) then
    lon1d@_FillValue = default_fillvalue(typeof(lon1d))
  end if

;Saving all lat/lon values into new 2d array based on U.S. states to use later
    if(ndm.gt.0) then
      lat1d_new(i,:)  = (where(data_mask_1d.eq.1,lat1d,lat1d@_FillValue))
      lon1d_new(i,:)  = (where(data_mask_1d.eq.1,lon1d,lon1d@_FillValue))

      end if

end do

print("Taking all non-missing lat/lon from all states into one array")
;Taking all non-missing lat/lon from all states into one array
lat1d_new2 = ndtooned(lat1d_new)
lon1d_new2 = ndtooned(lon1d_new)
lat1d_new3_ind = ind(.not.ismissing(lat1d_new2))
lon1d_new3_ind = ind(.not.ismissing(lon1d_new2))
lat1d_new3 = lat1d_new2(lat1d_new3_ind)
lon1d_new3 = lon1d_new2(lon1d_new3_ind)
;print(lat1d_new3)
;print(lon1d_new3)

;Now Finding respective ROW and COL indices to subset the data to these states
print("Now Finding respective ROW and COL indices to subset the data to these states")
ROW_ind = new((/dimsizes(lat1d_new3)/),integer)
COL_ind = new((/dimsizes(lat1d_new3)/),integer)

;---------updated, much faster loop
do a = 0, dimsizes(lat1d_new3) - 1
ROW_COL_ind= getind_latlon2d(lat2d,lon2d,lat1d_new3(a),lon1d_new3(a))
ROW_ind(a) = ROW_COL_ind(0,0)
COL_ind(a) = ROW_COL_ind(0,1)
end do
;---------------------------------


;ij = getind_latlon2d(lat2d,lon2d,(/lat_min,lat_max/),(/lon_min,lon_max/))
;;---Store to local variables for better code readability
  ilat1 = ij(0,0)
  ilat2 = ij(1,0)
  ilon1 = ij(0,1)
  ilon2 = ij(1,1)

print(ilat1)
print(ilat2)
print(ilon1)
print(ilon2)



;********************************
; plot
;********************************
   fo = "NH3_emis_hist_CB_shp_air"
   wks = gsn_open_wks("pdf","plots/"+fo )          ; open a workstation
   plots = new(1, graphic)
   gsn_define_colormap(wks,"precip3_16lev");(wks,"gui_default")  "BlAqGrYeOrRe" ; choose colormap

; to define the parameters that needed for the plotting
   res                 = True                    ; plot mods desired
   res@cnLinesOn       = False                   ; default is true;False, no contour lines will appear in the contour plot
   res@cnLineLabelsOn  = False                   ; turn off contour lines
   res@cnFillOn        = True                    ; color plot desired

; !!!!! any plot of data that is on a native grid, must use the "corners"
; method of zooming in on map.   
   res@mpLimitMode       = "Corners"            ; choose range of map
;   res@mpLeftCornerLatF  = lat2d(0,0)
;   res@mpLeftCornerLonF  = lon2d(0,0)
;   res@mpRightCornerLatF = lat2d(nlat-1,nlon-1)
;   res@mpRightCornerLonF = lon2d(nlat-1,nlon-1)

res@mpLeftCornerLatF  = lat2d(ilat1,ilon1)
   res@mpLeftCornerLonF  = lon2d(ilat1,ilon1)
   res@mpRightCornerLatF = lat2d(ilat2,ilon2)
   res@mpRightCornerLonF = lon2d(ilat2,ilon2)


; The following 4 pieces of information are REQUIRED to properly display
; data on a native lambert conformal grid.  This data should be specified
; somewhere in the model itself.      
   res@mpProjection        = "LambertConformal"
   res@mpLambertParallel1F = 33
   res@mpLambertParallel2F = 45
   res@mpLambertMeridianF  = -97

   res@mpGeophysicalLineColor = "Black"                   ; color of continental outlines
   res@mpPerimOn              = True                    ; draw box around map
   res@mpGridLineDashPattern  = 2                       ; lat/lon lines as dashed
   res@mpGridLineColor        = "Black"                 ; GridLine color
   res@mpOutlineBoundarySets = "GeophysicalAndUSStates" ; add state boundaries
   res@mpUSStateLineColor    = "Black"                    ; make them red

; usually, when data is placed onto a map, it is TRANSFORMED to the specified
; projection. Since this model is already on a native lambert conformal grid,
; we want to turn OFF the tranformation.
   res@tfDoNDCOverlay = True
   res@pmTickMarkDisplayMode = "Always"                 ; turn on tickmarks
   res@tmXTOn = False            ; turn off top   labels 
   res@tmYROn = False             ; turn off right labels
   res@tmXBOn = True            ; turn off bottom lables
   res@tmYLOn = True            ; turn off left lables

; panel plots
  res@gsnDraw                = False     ; do not draw
  res@gsnFrame               = False     ; do not advance frame

; Define tickmarks for the first two plots
;for the colorbar or called "tickmarks" in NCL
   res@cnLevelSelectionMode  = "ManualLevels"

;Resources for diff plots
   dres = res
   dres@cnMinLevelValF     = 0.0
   dres@cnMaxLevelValF     = 10.0
   dres@cnLevelSpacingF    = 1.0
   dres@cnFillPalette      = "precip3_16lev"

   res@cnMinLevelValF        = 40
   res@cnMaxLevelValF        = 70
   res@cnLevelSpacingF       = 5

dir_hist = "/work/MOD3APP/pcc/03.Chesbay.2050.Runs.Noah/emis/2011ec_v6_cb05_soa_ref_11f/US2_grid_window/"
dir_fut = "/work/MOD3APP/pcc/03.Chesbay.2050.Runs.Noah/emis/2040ei_v6_cb05v2_hdghgp2_ref_11g/"



;loop over the months
; months = (/"12","01","02","03","04","05","06","07","08","09","10","11"/)
 months = (/"J_D"/)
 nameofmonths = (/"Annual"/);  "Winter"/);,"Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov"/)

  nmonths = dimsizes (nameofmonths)

 do m = 0, nmonths-1
  ;6/23/14, zyq, add multi years files for monthly mean
  file_COUPLE = systemfunc (" ls -1 " + dir_hist + "/emis_mole_all_20111222_36US2_cmaq_cb05_soa_2011ec_v6_11f_"+months(m)+"_ave.nc")
  numFILES2 = dimsizes(file_COUPLE)
  print("numFILES = " + numFILES2)
  print(file_COUPLE)
  print (" ")

   f_COUPLE = addfile(file_COUPLE,"r")
   NH3_temp_COUPLE = f_COUPLE->NH3
   NH3_temp_COUPLE = -999
   NH3_temp_COUPLE_pop = f_COUPLE->NH3
   NH3_temp_COUPLE@_FillValue = -999       ; sets _FillValue to 0

;---------updated, much faster loop
print("Loop to put in CBW specific values...")
;Now Loop to put in CBW specific values
do a = 0, dimsizes(lat1d_new3) - 1
NH3_temp_COUPLE(:,:,ROW_ind(a),COL_ind(a))=NH3_temp_COUPLE_pop(:,:,ROW_ind(a),COL_ind(a))  ;Even Much faster loop
end do


   NH3_COUPLE = dim_avg_n_Wrap(NH3_temp_COUPLE(:,:,ilat1:ilat2,ilon1:ilon2-1),(/0,1/))

   printVarSummary(NH3_COUPLE)

  ;6/23/14, zyq, add multi years files for monthly mean
   file_OFFLINE = systemfunc (" ls -1 " + dir_fut + "/emis_mole_all_20111231_36US2_fill_cmaq_cb05v2_mplite_2040ei_v6_cb05v2_hdghgp2_ref_11g_"+months(m)+"_ave.nc")

  

  
   numFILES3 = dimsizes(file_OFFLINE)
   print("numFILES = " + numFILES3)
   print(file_OFFLINE)
   print (" ")

;  file_OFFLINE = dir_OFFLINE + "CCTM_ACONC.CMAQ-*_2050_OFFLINE.2050"+months(m)+".O3.hourly.combine.nc"
   f_OFFLINE = addfile(file_OFFLINE,"r")
   NH3_temp_OFFLINE = f_OFFLINE->NH3
   NH3_temp_OFFLINE = -999
   NH3_temp_OFFLINE_pop = f_OFFLINE->NH3
   NH3_temp_OFFLINE@_FillValue = -999           ; sets _FillValue to 0

;---------updated, much faster loop
print("Loop to put in CBW specific values...")
;Now Loop to put in CBW specific values
do a = 0, dimsizes(lat1d_new3) - 1
NH3_temp_OFFLINE(:,:,ROW_ind(a),COL_ind(a))=NH3_temp_OFFLINE_pop(:,:,ROW_ind(a),COL_ind(a))  ;Even Much faster loop
end do


   NH3_OFFLINE = dim_avg_n_Wrap(NH3_temp_OFFLINE(:,:,ilat1:ilat2,ilon1:ilon2-1),(/0,1/)) ;get two dimension

   printVarSummary(NH3_OFFLINE)

;create the map
; Working on titles
   res@tiMainString = "HIST "+"   "+"NH3"+"  (ppb)"
   res@tiMainFont   = 25
   res@tiMainFontHeightF = 0.02
;   plots(0) = gsn_csm_contour_map(wks,O3_COUPLE,res)    ; Draw contours over a map.

; for the right plot, different names
   res@tiMainString = "FUT "+"   "+"NH3 "+"  (ppb)"
;   plots(1) = gsn_csm_contour_map(wks,O3_OFFLINE,res)

  ;create the res for the difference map
  ; we only need to change the name, the vmin, vmax 
  ;for the colorbar or called "tickmarks" in NCL
   Diff = (NH3_COUPLE); - NH3_COUPLE)
   print("Total co-benefits Min and Max ") 
   printMinMax (Diff, True)
   ;working on titles
dres@lbOrientation =   "vertical"          ; vertical label bar   
dres@tiMainFont   = 25 
   dres@tiMainFontHeightF = 0.02
   dres@gsnCenterString = "Ave "+tostring(sprintf("%7.2f", avg(Diff)))
   dres@gsnRightString = "Max "+tostring(sprintf("%7.2f", max(Diff)))
   dres@gsnLeftString = "Min "+tostring(sprintf("%7.2f", min(Diff)))
   dres@tiMainString = nameofmonths(m)+ " Abs Diff FUT-HIST "+"NH3 "
;   dres@gsnCenterString = "mean value " + sum(Diff)/sum(USmask) + " ppb"
   ;gsn_define_colormap(wks,"BlWhRe")
   plots(m) = gsn_csm_contour_map(wks,Diff,dres)

;Relative Differences
  ;DiffRel = NO_OFFLINE/NO_COUPLE
  ;print("Min and Max for O3 relative changes Future/Base  ")
  ;printMinMax (DiffRel, True)

 ; dresrel = dres
 ; dresrel@cnLevelSelectionMode  = "ManualLevels"
 ; dresrel@cnMinLevelValF     := 0.9
 ; dresrel@cnMaxLevelValF     := 1.1
 ; dresrel@cnLevelSpacingF    := 0.02
 ; dresrel@cnFillPalette      = "BlWhRe"
  ;dresrel@gsnLeftString = "Min Diff "+tostring(min(DiffRel))
  ;dresrel@gsnRightString = "Max Diff "+tostring(max(DiffRel))

 ; dresrel@tiMainString = "Relative Diff Future/Base"

;  plots(3) =  gsn_csm_contour_map(wks,DiffRel,dresrel)

;end loop for the months
 end do 

;==============================================
; create panel
;=============================================
  pres                         = True
  pres@gsnPanelRowSpec         = True      ;tell panel what order to plot
  gsn_panel (wks, plots, (/1/), pres)
;  gsn_panel (wks, plots, (/3, 3, 3, 3/), pres)
  ;pres@gsnPanelCenter          = True      ;put the plot in center
  ;gsn_panel ()
  print("-------------------------------------------------------------------------")
  print("-------------------------------------------------------------------------")
;delete the parameter
  delete(file_COUPLE)
  delete(file_OFFLINE)
  delete(NH3_COUPLE)
  delete(NH3_OFFLINE)  
  delete(res)
  delete(dres)
  delete(wks)
  delete(plots)
  delete(pres)

end


