;*******************************************
; Written by Yuqiang Zhang, UNC, 2014
;
; Concepts illustrated:
;   - Plotting CMAQ data; also works for data using LCC project, like WRF and other RCM
;   - Drawing filled contours over a Lambert Conformal map
;   - Drawing U.S. states
;********************************************************************
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_code.ncl"   
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_csm.ncl"  
load "$NCARG_ROOT/lib/ncarg/nclscripts/wrf/WRF_contributed.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/contributed.ncl"
; This file still has to be loaded manually
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/shea_util.ncl"

;load ".hluresfile"
 
begin

scenarios = (/"historical","rcp4.5"/);,"rcp6.0","rcp8.5"/)
;scenarios = (/"rcp4.5"/)
nscenarios = dimsizes (scenarios)
var = "SWDNB"


   ;read the MCIP file and the 2D lon&lat
;   lonlatfile = "/asm/CLIMSIM/downscale/F30/task1/wrf/36NA/1995.v6/wrfout_d01_1996-01-01_00:00:00"
lonlatfile = "/work/MOD3APP/pcc/03.Chesbay.2050.Runs.Noah/rcp4.5/run/36NA/2050/wrfout_d01_2050-12-30_00:00:00" 
  flonlat = addfile(lonlatfile,"r")
   lon2d_temp = flonlat->XLONG
   lat2d_temp = flonlat->XLAT
   printVarSummary(lon2d_temp)
   lon2d = lon2d_temp(0,:,:)
   lat2d = lat2d_temp(0,:,:)
   printVarSummary(lon2d)
   printVarSummary(lat2d)
   lat1d = ndtooned(lat2d)
   lon1d = ndtooned(lon2d)
   nlatlon = dimsizes(lat1d)


;Subset the data and for Chesapeake Bay Region
;lat_min = 36.55993
;lat_max = 43.51854
;lon_min = -80.46683
;lon_max = -73.84467

;read in USGS Atlas Shapefile of U.S. States

fAin_shp = "Chesbay_N_Airshed.shp"
fA_shp   = addfile(fAin_shp,"r")
; Read data off shapefile
;
  geomDims      = getfilevardimsizes(fA_shp,"geometry")
  numFeatures   = geomDims(0)
  segments      = fA_shp->segments    ;U.S. segments
  geometry      = fA_shp->geometry    ;U.S. geometry
;  state_names   = fA_shp->STATE      ;U.S. state names
;  FIPS_names    = fA_shp->STATE_FIPS  ;U.S. state FIPS
  lat           = fA_shp->y           ;U.S. latitudes
  lon           = fA_shp->x           ;U.S. longitudes
  segsDims = dimsizes(segments)
  geomDims = dimsizes(geometry)
  latDims  = dimsizes(lat)
;
; Read global attributes
;
  geom_segIndex = fA_shp@geom_segIndex
  geom_numSegs  = fA_shp@geom_numSegs
  segs_xyzIndex = fA_shp@segs_xyzIndex
  segs_numPnts  = fA_shp@segs_numPnts
  layer_names   = fA_shp@layer_name


;ij = getind_latlon2d(lat2d,lon2d,(/lat_min,lat_max/),(/lon_min,lon_max/))
;---Store to local variables for better code readability
;  ilat1 = ij(0,0)
;  ilat2 = ij(1,0)
;  ilon1 = ij(0,1)
;  ilon2 = ij(1,1)

;print(ilat1)
;print(ilat2)
;print(ilon1)
;print(ilon2)

;get the dimensions
;   dims=dimsizes(lon2d(ilat1:ilat2,ilon1:ilon2))
;   print(dims)
;   ndims = dimsizes(dims)
;   nlon=dims(ndims-1)                                ; assign # lat/lon points
;   nlat=dims(ndims-2)
;   print(nlon)
;   print(nlat)



;------------This Section uses shapefile data to get the average of a data array over all points that fall within each(selected) state(s) in the U.S..

;Collecting all indices pertaining to selected U.S. States
State_ind = ind(layer_names.eq."Chesbay_N_Airshed")

;State_ind = ind(state_names.eq."Delaware")

;Make new 2d lat/long indice arrays to hold all new indices for lat/lons in selected U.S states
lat1d_new      = new((/dimsizes(State_ind),dimsizes(lat1d)/),float)
lon1d_new      = new((/dimsizes(State_ind),dimsizes(lon1d)/),float)
;print(dimsizes(lat2d_new_ind))

State_num = dimsizes(State_ind)
;print(State_num)
;print(State_ind)


;---Create array to hold new data mask, and set all values to 0 initially.

data_mask_1d = new(nlatlon,integer)
  data_mask_1d = 0
 skip_check   = new(nlatlon,integer)
  skip_check   = 0

do i=0, State_num-1
    print("--------------------------------------------------")
    print("Inspecting U.S. state '" + layer_names(State_ind(i)) + "'...")
data_mask_1d = 0    ; Be sure to reset to 0 for every state

;---Some States have multiple segments
    startSegment = geometry(State_ind(i), geom_segIndex)
    numSegments  = geometry(State_ind(i), geom_numSegs)
print("Some States have multiple segments: Segments = " + numSegments)


    do seg=startSegment, startSegment+numSegments-1
      startPT = segments(seg, segs_xyzIndex)
      endPT   = startPT + segments(seg, segs_numPnts) - 1

;---Loop through each point on data grid and see if it's in this state.
      do n=0,nlatlon-1
;        nn = ii_latlon(n)     ; Get index of point we're checking

      ; This "if" statement speeds up code, by making sure we don't
; needlessly check a lat/lon point if:
;
; - we've already found it in another state
; - it doesn't fall within the general lat/lon box that covers this state
;
        if(skip_check(n).or.\
           lat1d(n).lt.min(lat(startPT:endPT)).or.\
           lat1d(n).gt.max(lat(startPT:endPT)).or.\
           lon1d(n).lt.min(lon(startPT:endPT)).or.\
           lon1d(n).gt.max(lon(startPT:endPT)))
           continue
        end if


;---Here's the check if the latitude point is in the state.
        if(gc_inout(lat1d(n),lon1d(n),\
                    lat(startPT:endPT),lon(startPT:endPT))) then
          data_mask_1d(n) = 1    ; This point is inside this state
          skip_check(n)   = 1    ; Don't check this point state
;         print(lat1d(n))
;         print(lon1d(n))
        end if


      end do
    end do    ; End of collecting points for this state

;---Count number of points found in this state
;print (data_mask_1d)
    ndm = num(data_mask_1d.eq.1)
    print(ndm + " data values found in Chesbay.")


; If there are values found in this state, then take new lat/lon indices to use for the PDF calculation below.
;

;---First Make sure "lat1d" has a missing value.
  if(.not.isatt(lat1d,"_FillValue")) then
    lat1d@_FillValue = default_fillvalue(typeof(lat1d))
  end if

;---First Make sure "lon1d" has a missing value.
  if(.not.isatt(lon1d,"_FillValue")) then
    lon1d@_FillValue = default_fillvalue(typeof(lon1d))
  end if

;Saving all lat/lon values into new 2d array based on U.S. states to use later
    if(ndm.gt.0) then
      lat1d_new(i,:)  = (where(data_mask_1d.eq.1,lat1d,lat1d@_FillValue))
      lon1d_new(i,:)  = (where(data_mask_1d.eq.1,lon1d,lon1d@_FillValue))

      end if

end do 

print("Taking all non-missing lat/lon from all states into one array")
;Taking all non-missing lat/lon from all states into one array
lat1d_new2 = ndtooned(lat1d_new)
lon1d_new2 = ndtooned(lon1d_new)
lat1d_new3_ind = ind(.not.ismissing(lat1d_new2))
lon1d_new3_ind = ind(.not.ismissing(lon1d_new2))
lat1d_new3 = lat1d_new2(lat1d_new3_ind)
lon1d_new3 = lon1d_new2(lon1d_new3_ind)
;print(lat1d_new3)
;print(lon1d_new3)

;Now Finding respective ROW and COL indices to subset the data to these states
print("Now Finding respective ROW and COL indices to subset the data to these states")
ROW_ind = new((/dimsizes(lat1d_new3)/),integer)
COL_ind = new((/dimsizes(lat1d_new3)/),integer)

;---------updated, much faster loop
do a = 0, dimsizes(lat1d_new3) - 1
ROW_COL_ind= getind_latlon2d(lat2d,lon2d,lat1d_new3(a),lon1d_new3(a))
ROW_ind(a) = ROW_COL_ind(0,0)
COL_ind(a) = ROW_COL_ind(0,1)
end do
;---------------------------------

;********************************
; plot
;********************************
 setvalues NhlGetWorkspaceObjectId()
    "wsMaximumSize" : 300000000
   end setvalues

  res            = True                         ; plot mods desired
  res@trYMinF = 181.
  res@trYMaxF = 194.

   ;loop over the scenarios and years
print("entering loops")
 do n = 0, nscenarios - 1

if(scenarios(n).eq."historical") then
years = (/"1995","1996","1997","1998","1999","2000","2001","2002","2003","2004"/)
years_int= (/1995,1996,1997,1998,1999,2000,2001,2002,2003,2004/)
;years = (/"1995","1996","1997","1998","1999"/)
;years_int = (/1995,1996,1997,1998,1999/)
else
years = (/"2045","2046","2047","2048","2049","2050","2051","2052","2053","2054"/)
years_int= (/2045,2046,2047,2048,2049,2050,2051,2052,2053,2054/)
;years = (/"2048","2049","2050"/)
;years_int= (/2048,2049,2050/)
end if 
myears = dimsizes (years)
months = (/"01","02","03","04","05","06","07","08","09","10","11","12"/)
nmonths=dimsizes(months)

fo = scenarios(n)+"_"+var+"_barchart_annual_mean_CB_new_shp_air"
wks = gsn_open_wks("pdf","plots/"+fo )          ; open a workstation
res@gsnXYBarChart = True            ; Create bar plot
;Set data arrays
SWDNB_1D_Data = new((/nscenarios,myears/),float)  ;For barchart
x = ispan(1,myears,1)
SWDNB_1D = new((/nscenarios,myears/),float)  ;For 10 year average of the mean
SWDNB_2D = new((/nscenarios,myears,nmonths/),float)  ;For 10 year average of the mean

do m = 0, myears-1
do s = 0, nmonths-1
;dir= "/work/MOD3APP/pcc/03.Chesbay.2050.Runs.Noah/"+scenarios(n)+"/run/36NA/annual/"
dir= "/work/MOD3APP/pcc/03.Chesbay.2050.Runs.Noah/"+scenarios(n)+"/run/36NA/postp/wrfextr4/all/"
print(dir)

;  filen = dir + var + "_d01_" +years(m) + ".nc"
   filen = dir + "wrfextr" + "_d01_" +years(m) + "-" + months(s) + ".nc"

  print(filen)
  print (" ")

   f = addfile(filen,"r")
   ;SWDNB = f->SWDNB(:,ilat1:ilat2,ilon1:ilon2)
   SWDNB = f->SWDNB
   SWDNB = -999
   SWDNB_pop = f->SWDNB        ;For populating with CBW specific values
   SWDNB@_FillValue = -999  ;Fill with missing values
   ;---------updated, much faster loop
print("Loop to put in CBW specific values...")
;Now Loop to put in CBW specific values
do a = 0, dimsizes(lat1d_new3) - 1
SWDNB(:,ROW_ind(a),COL_ind(a))=SWDNB_pop(:,ROW_ind(a),COL_ind(a))  ;Even Much faster loop
end do
;-------------------------------
   SWDNB_2D(n,m,s) = avg(SWDNB) ;Take spatial and time average - Produces single yearly value
   print(years(m) + " " + months(s) + " mean = " + SWDNB_2D(n,m,s))

delete(SWDNB)
delete(SWDNB_pop)
end do
SWDNB_1D(n,m) = avg(SWDNB_2D(n,m,:))
print(years(m) + " avg = " + SWDNB_1D(n,m))

;end loop for the years 
end do 

print(scenarios(n)+" standard deviation = ")
SWDNB_1D_stddev = stddev(SWDNB_1D(n,:))
print(SWDNB_1D_stddev)



;Set reference line and color the bar chart
res@gsnYRefLine = dim_avg(SWDNB_1D(n,:))
res@gsnAboveYRefLineColor = "red"           ; above ref line fill red
res@gsnBelowYRefLineColor = "blue"          ; below ref line fill blue


; Working on titles
   res@tiMainString = scenarios(n)+"_"+years(0)+"-"+years(myears-1)+"_"+var+"_mean (W/m2)"
   res@tiMainFont   = 25
   res@tiMainFontHeightF = 0.02
   res@tmXBLabelFontHeightF = 0.015

;For plotting years as integers in bar chart
;years_int = stringtointeger(years)

  res@tmXBMode        = "Explicit"	
  res@tmXBValues      = years_int
  res@tmXBLabels      = "" + res@tmXBValues 

plot = gsn_csm_xy(wks,years_int,SWDNB_1D(n,:),res)	

  draw(wks)                                    ; boxplot does not call these
  frame(wks)                                   ; for you


delete(wks)

system("convert   -trim  -page A4+0+0 plots/"+fo+".pdf plots/"+fo+".png")
delete(years)
delete(years_int)
delete(SWDNB_1D_Data)
delete(SWDNB_1D)
delete(x)
delete(res@tmXBValues)
delete(res@tmXBLabels)
end do ;end loop for the scenarios
end


