;*******************************************
; Written by Yuqiang Zhang, UNC, 2014
;
; Concepts illustrated:
;   - Plotting CMAQ data; also works for data using LCC project, like WRF and other RCM
;   - Drawing filled contours over a Lambert Conformal map
;   - Drawing U.S. states
;********************************************************************
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_code.ncl"   
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_csm.ncl"  
load "$NCARG_ROOT/lib/ncarg/nclscripts/wrf/WRF_contributed.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/contributed.ncl"
;load ".hluresfile"
 
begin

   ;read the MCIP file and the 2D lon&lat
;   lonlatfile = "/asm/CLIMSIM/downscale/F30/task1/wrf/36NA/1995.v6/wrfout_d01_1996-01-01_00:00:00"
   lonlatfile = "/work/MOD3APP/pcc/03.Chesbay.2050.Runs.Noah/rcp4.5/run/36NA/2050/wrfout_d01_2050-12-30_00:00:00"
   flonlat = addfile(lonlatfile,"r")
   lon2d_temp = flonlat->XLONG
   lat2d_temp = flonlat->XLAT
   printVarSummary(lon2d_temp)
   lon2d = lon2d_temp(0,:,:)
   lat2d = lat2d_temp(0,:,:)
   printVarSummary(lon2d)
   printVarSummary(lat2d)
   lat1d = ndtooned(lat2d)
   lon1d = ndtooned(lon2d)
   nlatlon = dimsizes(lat1d)

;Whole Domain
;lat_min = min(ndtooned(lat2d))
;lat_max = max(ndtooned(lat2d))
;lon_min = min(ndtooned(lon2d))
;lon_max = max(ndtooned(lon2d))

;Subset the data/map for around Chesapeake Bay Region
lat_min = 33.
lat_max = 46.
lon_min = -88.
lon_max = -69.

;read in USGS Atlas Shapefile of U.S. States

fAin_shp = "Chesbay_N_Airshed.shp"
fA_shp   = addfile(fAin_shp,"r")
; Read data off shapefile
;
  geomDims      = getfilevardimsizes(fA_shp,"geometry")
  numFeatures   = geomDims(0)
  segments      = fA_shp->segments    ;U.S. segments
  geometry      = fA_shp->geometry    ;U.S. geometry
;  state_names   = fA_shp->STATE      ;U.S. state names
;  FIPS_names    = fA_shp->STATE_FIPS  ;U.S. state FIPS
  lat           = fA_shp->y           ;U.S. latitudes
  lon           = fA_shp->x           ;U.S. longitudes
  segsDims = dimsizes(segments)
  geomDims = dimsizes(geometry)
  latDims  = dimsizes(lat)
;
; Read global attributes
;
  geom_segIndex = fA_shp@geom_segIndex
  geom_numSegs  = fA_shp@geom_numSegs
  segs_xyzIndex = fA_shp@segs_xyzIndex
  segs_numPnts  = fA_shp@segs_numPnts
  layer_names   = fA_shp@layer_name


ij = getind_latlon2d(lat2d,lon2d,(/lat_min,lat_max/),(/lon_min,lon_max/))
;---Store to local variables for better code readability
  ilat1 = ij(0,0)
  ilat2 = ij(1,0)
  ilon1 = ij(0,1)
  ilon2 = ij(1,1)

;print(ilat1)
;print(ilat2)
;print(ilon1)
;print(ilon2)

;get the dimensions
;   dims=dimsizes(lon2d(ilat1:ilat2,ilon1:ilon2))
;   print(dims)
;   ndims = dimsizes(dims)
;   nlon=dims(ndims-1)                                ; assign # lat/lon points
;   nlat=dims(ndims-2)
;   print(nlon)
;   print(nlat)



;------------This Section uses shapefile data to get the average of a data array over all points that fall within each(selected) state(s) in the U.S..

;Collecting all indices pertaining to selected U.S. States
State_ind = ind(layer_names.eq."Chesbay_N_Airshed")

;State_ind = ind(state_names.eq."Delaware")

;Make new 2d lat/long indice arrays to hold all new indices for lat/lons in selected U.S states
lat1d_new      = new((/dimsizes(State_ind),dimsizes(lat1d)/),float)
lon1d_new      = new((/dimsizes(State_ind),dimsizes(lon1d)/),float)
;print(dimsizes(lat2d_new_ind))

State_num = dimsizes(State_ind)

;print(State_num)
;print(State_ind)


;---Create array to hold new data mask, and set all values to 0 initially.

data_mask_1d = new(nlatlon,integer)
  data_mask_1d = 0
 skip_check   = new(nlatlon,integer)
  skip_check   = 0

do i=0, State_num-1
    print("--------------------------------------------------")
    print("Inspecting U.S. state '" + layer_names(State_ind(i)) + "'...")
data_mask_1d = 0    ; Be sure to reset to 0 for every state

;---Some States have multiple segments
    startSegment = geometry(State_ind(i), geom_segIndex)
    numSegments  = geometry(State_ind(i), geom_numSegs)
print("Some States have multiple segments: Segments = " + numSegments)


    do seg=startSegment, startSegment+numSegments-1
      startPT = segments(seg, segs_xyzIndex)
      endPT   = startPT + segments(seg, segs_numPnts) - 1

;---Loop through each point on data grid and see if it's in this state.
      do n=0,nlatlon-1
;        nn = ii_latlon(n)     ; Get index of point we're checking

      ; This "if" statement speeds up code, by making sure we don't
; needlessly check a lat/lon point if:
;
; - we've already found it in another state
; - it doesn't fall within the general lat/lon box that covers this state
;
        if(skip_check(n).or.\
           lat1d(n).lt.min(lat(startPT:endPT)).or.\
           lat1d(n).gt.max(lat(startPT:endPT)).or.\
           lon1d(n).lt.min(lon(startPT:endPT)).or.\
           lon1d(n).gt.max(lon(startPT:endPT)))
           continue
        end if


;---Here's the check if the latitude point is in the state.
        if(gc_inout(lat1d(n),lon1d(n),\
                    lat(startPT:endPT),lon(startPT:endPT))) then
          data_mask_1d(n) = 1    ; This point is inside this state
          skip_check(n)   = 1    ; Don't check this point state
;         print(lat1d(n))
;         print(lon1d(n))
        end if


      end do
    end do    ; End of collecting points for this state

;---Count number of points found in this state
;print (data_mask_1d)
    ndm = num(data_mask_1d.eq.1)
    print(ndm + " data values found in Chesbay.")

; If there are values found in this state, then take new lat/lon indices to use for the PDF calculation below.
;

;---First Make sure "lat1d" has a missing value.
  if(.not.isatt(lat1d,"_FillValue")) then
    lat1d@_FillValue = default_fillvalue(typeof(lat1d))
  end if

;---First Make sure "lon1d" has a missing value.
  if(.not.isatt(lon1d,"_FillValue")) then
    lon1d@_FillValue = default_fillvalue(typeof(lon1d))
  end if

;Saving all lat/lon values into new 2d array based on U.S. states to use later
    if(ndm.gt.0) then
      lat1d_new(i,:)  = (where(data_mask_1d.eq.1,lat1d,lat1d@_FillValue))
      lon1d_new(i,:)  = (where(data_mask_1d.eq.1,lon1d,lon1d@_FillValue))

      end if

end do

print("Taking all non-missing lat/lon from all states into one array")
;Taking all non-missing lat/lon from all states into one array
lat1d_new2 = ndtooned(lat1d_new)
lon1d_new2 = ndtooned(lon1d_new)
lat1d_new3_ind = ind(.not.ismissing(lat1d_new2))
lon1d_new3_ind = ind(.not.ismissing(lon1d_new2))
lat1d_new3 = lat1d_new2(lat1d_new3_ind)
lon1d_new3 = lon1d_new2(lon1d_new3_ind)
;print(lat1d_new3)
;print(lon1d_new3)

;Now Finding respective ROW and COL indices to subset the data to these states
print("Now Finding respective ROW and COL indices to subset the data to these states")
ROW_ind = new((/dimsizes(lat1d_new3)/),integer)
COL_ind = new((/dimsizes(lat1d_new3)/),integer)

;---------updated, much faster loop
do a = 0, dimsizes(lat1d_new3) - 1
ROW_COL_ind= getind_latlon2d(lat2d,lon2d,lat1d_new3(a),lon1d_new3(a))
ROW_ind(a) = ROW_COL_ind(0,0)
COL_ind(a) = ROW_COL_ind(0,1)
end do
;---------------------------------


;ij = getind_latlon2d(lat2d,lon2d,(/lat_min,lat_max/),(/lon_min,lon_max/))
;;---Store to local variables for better code readability
  ilat1 = ij(0,0)
  ilat2 = ij(1,0)
  ilon1 = ij(0,1)
  ilon2 = ij(1,1)

print(ilat1)
print(ilat2)
print(ilon1)
print(ilon2)

;;get the dimensions
;   dims=dimsizes(lon2d)
;   print(dims)
;   ndims = dimsizes(dims)
;   nlon=dims(ndims-1)                                ; assign # lat/lon points
;   nlat=dims(ndims-2)
;   print(nlon)
;   print(nlat)






;********************************
; plot
;********************************
   fo = "Q2_plots_36KM_fut_hist_CBPO_monthly_annual_values_tot_CB_shp_air"
   wks = gsn_open_wks("pdf","plots/"+fo )          ; open a workstation
   
   setvalues NhlGetWorkspaceObjectId()
    "wsMaximumSize" : 300000000
   end setvalues

   plots = new(13, graphic)
   gsn_define_colormap(wks,"BlAqGrYeOrReVi200");(wks,"gui_default")  "BlAqGrYeOrRe" ; choose colormap

; to define the parameters that needed for the plotting
   res                 = True                    ; plot mods desired
   res@cnLinesOn       = False                   ; default is true;False, no contour lines will appear in the contour plot
   res@cnLineLabelsOn  = False                   ; turn off contour lines
   res@cnFillOn        = True                    ; color plot desired

; !!!!! any plot of data that is on a native grid, must use the "corners"
; method of zooming in on map.   
   res@mpLimitMode       = "Corners"            ; choose range of map
;   res@mpLeftCornerLatF  = lat2d(0,0)
;   res@mpLeftCornerLonF  = lon2d(0,0)
;   res@mpRightCornerLatF = lat2d(nlat-1,nlon-1)
;   res@mpRightCornerLonF = lon2d(nlat-1,nlon-1)


res@mpLeftCornerLatF  = lat2d(ilat1,ilon1)
   res@mpLeftCornerLonF  = lon2d(ilat1,ilon1)
   res@mpRightCornerLatF = lat2d(ilat2,ilon2)
   res@mpRightCornerLonF = lon2d(ilat2,ilon2)


; The following 4 pieces of information are REQUIRED to properly display
; data on a native lambert conformal grid.  This data should be specified
; somewhere in the model itself.      
   res@mpProjection        = "LambertConformal"
   res@mpLambertParallel1F = 33
   res@mpLambertParallel2F = 45
   res@mpLambertMeridianF  = -97

   res@mpGeophysicalLineColor = "Black"                   ; color of continental outlines
   res@mpPerimOn              = True                    ; draw box around map
   res@mpGridLineDashPattern  = 2                       ; lat/lon lines as dashed
   res@mpGridLineColor        = "Black"                 ; GridLine color
   res@mpOutlineBoundarySets = "GeophysicalAndUSStates" ; add state boundaries
   res@mpUSStateLineColor    = "Black"                    ; make them red

; usually, when data is placed onto a map, it is TRANSFORMED to the specified
; projection. Since this model is already on a native lambert conformal grid,
; we want to turn OFF the tranformation.
   res@tfDoNDCOverlay = True
   res@pmTickMarkDisplayMode = "Always"                 ; turn on tickmarks
   res@tmXTOn = False            ; turn off top   labels 
   res@tmYROn = False             ; turn off right labels
   res@tmXBOn = True            ; turn off bottom lables
   res@tmYLOn = True            ; turn off left lables

; panel plots
  res@gsnDraw                = False     ; do not draw
  res@gsnFrame               = False     ; do not advance frame

; Define tickmarks for the first two plots

   res@cnLevelSelectionMode  = "ManualLevels"

;Resources for diff plots
   dres = res
   dres@cnMinLevelValF     =  0.
   dres@cnMaxLevelValF     =  15.
   dres@cnLevelSpacingF    =  0.5
   dres@cnFillPalette      = "BlAqGrYeOrReVi200"

   res@cnMinLevelValF        = 1
   res@cnMaxLevelValF        = 10
   res@cnLevelSpacingF       = 1

;Current Year Tests
dir_NOAH_BASE = "/work/MOD3APP/pcc/03.Chesbay.2050.Runs.Noah/historical/run/36NA/postp/wrfextr4/"

;Future Year Tests
dir_COUPLE5 = "/work/MOD3APP/pcc/03.Chesbay.2050.Runs.Noah/rcp4.5/run/36NA/postp/wrfextr4/"

  ;loop over the months
months = (/"12","01","02","03","04","05","06","07","08","09","10","11","annual"/)
nameofmonths = (/"December","January","February","March","April","May","June","July","August","September","October","November","Annual"/)
nmonths = dimsizes (months)
;;Analysis after spinup
;s_anal = 119
;e_anal = 263

 do m = 0, nmonths-1
;Noah Base
print("name of month = "+months(m))  
   if (months(m).eq."annual") then
   file_NOAH_BASE1 = systemfunc (" ls -1 " + dir_NOAH_BASE + "/wrfextr_d01_1995_2004."+months(m)+".avg.nc") 
   else
   file_NOAH_BASE1 = systemfunc (" ls -1 " + dir_NOAH_BASE + "/wrfextr_d01_1995_2004-"+months(m)+".avg.nc")

   end if

   f_NOAH_BASE1 = addfile(file_NOAH_BASE1,"r")
    Q2_temp_NOAH_BASE1 = f_NOAH_BASE1->Q2
    Q2_temp_NOAH_BASE1 = -999
    Q2_temp_NOAH_BASE1_pop = f_NOAH_BASE1->Q2
    Q2_temp_NOAH_BASE1@_FillValue = -999  ;Fill with missing values

;---------updated, much faster loop
print("Loop to put in CBW specific values...")
;Now Loop to put in CBW specific values
do a = 0, dimsizes(lat1d_new3) - 1
Q2_temp_NOAH_BASE1(:,ROW_ind(a),COL_ind(a))=Q2_temp_NOAH_BASE1_pop(:,ROW_ind(a),COL_ind(a))  ;Even Much faster loop
end do

   Q2_NOAH_BASE1 = dim_avg_n_Wrap(Q2_temp_NOAH_BASE1(:,ilat1:ilat2,ilon1:ilon2-1),(/0/)) ;take average and use only subsetted data/map

   Q2_NOAH_BASE = Q2_NOAH_BASE1 * 1000.0

   printVarSummary(Q2_NOAH_BASE)

  if (months(m).eq."annual") then
  file_COUPLE51 = systemfunc (" ls -1 " + dir_COUPLE5 + "/wrfextr_d01_2045_2054."+months(m)+".avg.nc")
  else
  file_COUPLE51 = systemfunc (" ls -1 " + dir_COUPLE5 + "/wrfextr_d01_2045_2054-"+months(m)+".avg.nc")
  end if
  

   f_COUPLE51 = addfile(file_COUPLE51,"r")
   Q2_temp_COUPLE51 = f_COUPLE51->Q2
   Q2_temp_COUPLE51 = -999
   Q2_temp_COUPLE51_pop = f_COUPLE51->Q2
   Q2_temp_COUPLE51@_FillValue = -999  ;Fill with missing values 

   ;---------updated, much faster loop
print("Loop to put in CBW specific values...")
;Now Loop to put in CBW specific values
do a = 0, dimsizes(lat1d_new3) - 1
Q2_temp_COUPLE51(:,ROW_ind(a),COL_ind(a))=Q2_temp_COUPLE51_pop(:,ROW_ind(a),COL_ind(a))  ;Even Much faster loop
end do

   Q2_COUPLE51 = dim_avg_n_Wrap(Q2_temp_COUPLE51(:,ilat1:ilat2,ilon1:ilon2-1),(/0/))  ;take average and use only subsetted data/map 
 
   Q2_COUPLE5 = Q2_COUPLE51 * 1000.0

   printVarSummary(Q2_COUPLE5)

;Q2_Data = (Q2_COUPLE5-Q2_NOAH_BASE);/abs(Q2_NOAH_BASE))*100.0
Q2_Data = Q2_NOAH_BASE

;Q2_Data = (/Q2_NOAH_BASE,Q2_COUPLE5/)
;Q2_Data = (/Q2_COUPLE1,Q2_COUPLE2,Q2_COUPLE3,Q2_COUPLE4,Q2_COUPLE5/)

printVarSummary(Q2_Data)

;nscenarios = 2
;nscenarios = 5

nameofscenarios = (/"RCP4.5 - Historical"/)
;nameofscenarios = (/"Noah_Dev","Noah_Mosaic", "Noah_Mosaic_veg","Noah_Mosaic_soilveg","Noah_Mosaic_Thin_soilveg"/)

;do n = 0, nscenarios-1
dres@gsnCenterStringFontHeightF =.015
dres@gsnRightStringFontHeightF =.015
  dres@gsnLeftStringFontHeightF =.015
   
  
   Data = Q2_Data   ;(n,:,:)
   print("Min and Max ") 
   printMinMax (Data, True)
   print("Avg. ")
   print( avg(Data))
   dres@lbLabelFontHeightF =.022
   dres@tiMainFont   = 25 
   dres@tiMainFontHeightF = 0.025
   dres@gsnLeftString = "Min "+tostring(sprintf("%7.2f", min(Data)))
   dres@gsnRightString = "Max "+tostring(sprintf("%7.2f", max(Data)))
   dres@gsnCenterString = "Avg "+tostring(sprintf("%7.2f", avg(Data)))
   dres@tiMainString = nameofmonths(m) 
    dres@lbOrientation = "vertical"
    plots(m) = gsn_csm_contour_map(wks,Data,dres)
   
;end loop for the months and scenarios
end do 
;end do
;==============================================
; create panel
;=============================================
  pres                         = True
  pres@gsnPanelRowSpec         = True      ;tell panel what order to plot
; pres@gsnPanelMainString = "Q2 "+" Absolute Values (K)"     ; new resource added in NCL V6.4.0
  pres@txString           = "Q2 "+" Absolute Values (K)"     ; use this for NCL V6.3.0 and earlier


  gsn_panel (wks, plots, (/4, 4, 4, 4/), pres)
  ;pres@gsnPanelCenter          = True      ;put the plot in center
  ;gsn_panel ()
  print("-------------------------------------------------------------------------")
  print("-------------------------------------------------------------------------")
;delete the parameter
;  delete(file_COUPLE)
;  delete(file_COUPLE)
;  delete(Q2_COUPLE)
;  delete(Q2_COUPLE)  
  delete(res)
  delete(dres)
  delete(wks)
  delete(plots)
  delete(pres)
system("convert   -trim  -page A4+0+0 plots/"+fo+".pdf plots/"+fo+".png")
end


