load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_code.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_csm.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/contributed.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/wrf/WRF_contributed.ncl"
begin
;Created by Patrick Campbell

data_plots      = new((/2,12/),double)  ;100 is based on chosen NCL number of PDF bins of 100
var  = "L1_ANH3_L2_ANH3"
years_setA = (/"01","02","03","04","05","06","07","08","09","10","11","12"/)
scenariosA = (/"CURR","FUT_CLIM"/)

;Directory of obs,simuluation
fB_dir = "/work/MOD3APP/pcc/03.Chesbay.2050.Runs.Noah/EPIC/CURR/"
fC_dir = "/work/MOD3APP/pcc/03.Chesbay.2050.Runs.Noah/EPIC/FUT_CLIM/"

;Directory of map files defining WRF grid:
map_dir = "/work/MOD3APP/pcc/03.Chesbay.2050.Runs.Noah/rcp4.5/run/36NA/2045/MCIP/"


;read in map file defining WRF grid:
mapin = map_dir+"GRIDCRO2D_cesm36k_20450822"
fA_map=addfile(mapin,"r")


; Get every lat/lon pair of the original data array, and
; turn them into 1D arrays.  We will use these 1D arrays
; when checking which lat/lon pairs fall into each region of U.S. 

lat2d = fA_map->LAT(0,0,:,:)
lon2d = fA_map->LON(0,0,:,:)
lat1d = ndtooned(lat2d)
lon1d = ndtooned(lon2d)
nlatlon = dimsizes(lat1d)
;print(lat2d)

;read in USGS Atlas Shapefile of U.S. States

fAin_shp = "Chesbay_N_Airshed.shp"
fA_shp   = addfile(fAin_shp,"r")
; Read data off shapefile
;
  geomDims      = getfilevardimsizes(fA_shp,"geometry")
  numFeatures   = geomDims(0)
  segments      = fA_shp->segments    ;U.S. segments
  geometry      = fA_shp->geometry    ;U.S. geometry
;  state_names   = fA_shp->STATE      ;U.S. state names
;  FIPS_names    = fA_shp->STATE_FIPS  ;U.S. state FIPS
  lat           = fA_shp->y           ;U.S. latitudes
  lon           = fA_shp->x           ;U.S. longitudes
  segsDims = dimsizes(segments)
  geomDims = dimsizes(geometry)
  latDims  = dimsizes(lat)
;
; Read global attributes
;
  geom_segIndex = fA_shp@geom_segIndex
  geom_numSegs  = fA_shp@geom_numSegs
  segs_xyzIndex = fA_shp@segs_xyzIndex
  segs_numPnts  = fA_shp@segs_numPnts
  layer_names   = fA_shp@layer_name

;print (segments)
;print (geometry)
;print (state_names)
;print (FIPS_names)
;print (lat)
;print (lon)


;------------This Section uses shapefile data to get the average of a data array over all points that fall within each(selected) state(s) in the U.S..

;Collecting all indices pertaining to selected U.S. States
State_ind = ind(layer_names.eq."Chesbay_N_Airshed")

;State_ind = ind(state_names.eq."Delaware")

;Make new 2d lat/long indice arrays to hold all new indices for lat/lons in selected U.S states
lat1d_new      = new((/dimsizes(State_ind),dimsizes(lat1d)/),float)
lon1d_new      = new((/dimsizes(State_ind),dimsizes(lon1d)/),float)
;print(dimsizes(lat2d_new_ind))

State_num = dimsizes(State_ind)
;print(State_num)
;print(State_ind)


;---Create array to hold new data mask, and set all values to 0 initially.

data_mask_1d = new(nlatlon,integer)
  data_mask_1d = 0 
 skip_check   = new(nlatlon,integer)
  skip_check   = 0

;print(data_mask_1d)

; This is the loop that loops across each selected state of U.S.,
; and collects all the lat/lon points that are inside that
; state (data_mask_1d=1).

do i=0, State_num-1
    print("--------------------------------------------------")
    print("Inspecting U.S. state '" + layer_names(State_ind(i)) + "'...")
data_mask_1d = 0    ; Be sure to reset to 0 for every state

;---Some States have multiple segments
    startSegment = geometry(State_ind(i), geom_segIndex)
    numSegments  = geometry(State_ind(i), geom_numSegs)
print("Some States have multiple segments: Segments = " + numSegments)


    do seg=startSegment, startSegment+numSegments-1
      startPT = segments(seg, segs_xyzIndex)
      endPT   = startPT + segments(seg, segs_numPnts) - 1
    
;---Loop through each point on data grid and see if it's in this state.
      do n=0,nlatlon-1
;        nn = ii_latlon(n)     ; Get index of point we're checking
    
      ; This "if" statement speeds up code, by making sure we don't
; needlessly check a lat/lon point if:
; 
; - we've already found it in another state
; - it doesn't fall within the general lat/lon box that covers this state
; 
        if(skip_check(n).or.\
           lat1d(n).lt.min(lat(startPT:endPT)).or.\
           lat1d(n).gt.max(lat(startPT:endPT)).or.\
           lon1d(n).lt.min(lon(startPT:endPT)).or.\
           lon1d(n).gt.max(lon(startPT:endPT))) 
           continue
        end if
        

;---Here's the check if the latitude point is in the state.
        if(gc_inout(lat1d(n),lon1d(n),\
                    lat(startPT:endPT),lon(startPT:endPT))) then
          data_mask_1d(n) = 1    ; This point is inside this state
          skip_check(n)   = 1    ; Don't check this point state
;         print(lat1d(n))
;         print(lon1d(n)) 
        end if


      end do
    end do    ; End of collecting points for this state

;---Count number of points found in this state
;print (data_mask_1d)
    ndm = num(data_mask_1d.eq.1) 
    print(ndm + " data values found in Chesbay.")


; If there are values found in this state, then take new lat/lon indices to use for the PDF calculation below.
;

;---First Make sure "lat1d" has a missing value.
  if(.not.isatt(lat1d,"_FillValue")) then
    lat1d@_FillValue = default_fillvalue(typeof(lat1d))
  end if

;---First Make sure "lon1d" has a missing value.
  if(.not.isatt(lon1d,"_FillValue")) then
    lon1d@_FillValue = default_fillvalue(typeof(lon1d))
  end if

;Saving all lat/lon values into new 2d array based on U.S. states to use later
    if(ndm.gt.0) then
      lat1d_new(i,:)  = (where(data_mask_1d.eq.1,lat1d,lat1d@_FillValue))
      lon1d_new(i,:)  = (where(data_mask_1d.eq.1,lon1d,lon1d@_FillValue))
     
      end if

;     print(lat1d_new(i,:))
;     print(lon1d_new(i,:))

end do

print("Taking all non-missing lat/lon from all states into one array")
;Taking all non-missing lat/lon from all states into one array
lat1d_new2 = ndtooned(lat1d_new)
lon1d_new2 = ndtooned(lon1d_new)
lat1d_new3_ind = ind(.not.ismissing(lat1d_new2))
lon1d_new3_ind = ind(.not.ismissing(lon1d_new2))
lat1d_new3 = lat1d_new2(lat1d_new3_ind)
lon1d_new3 = lon1d_new2(lon1d_new3_ind)
;print(lat1d_new3)
;print(lon1d_new3)

;Now Finding respective ROW and COL indices to subset the data to these states
print("Now Finding respective ROW and COL indices to subset the data to these states")
ROW_ind = new((/dimsizes(lat1d_new3)/),integer)
COL_ind = new((/dimsizes(lat1d_new3)/),integer)

;---------updated, much faster loop
do a = 0, dimsizes(lat1d_new3) - 1
ROW_COL_ind= getind_latlon2d(lat2d,lon2d,lat1d_new3(a),lon1d_new3(a))
ROW_ind(a) = ROW_COL_ind(0,0)
COL_ind(a) = ROW_COL_ind(0,1)
end do
;---------------------------------

;---This Section Calculates the Cumulative PDF of MDA8 Ozone
do i = 0, dimsizes(scenariosA) -1
do j = 0, dimsizes(years_setA) -1 

year_setA = years_setA(j)
scenarioA = scenariosA(i)
print (var)
print (year_setA)
print (scenarioA)

;read in 1st CMAQ simuluation file (single day, month, year, or multiple year average file)


if(scenarioA.eq."CURR") then

fAin = fB_dir + "EPIC_"+var+".sum_1995_2004_ave_"+year_setA + ".nc"
fA=addfile(fAin,"r")


;This is subsetted data for selected states
print("Subsetting data for selected states...")

xA = fA->L2_ANH3
xA = -999
xB = fA->L2_ANH3
xA@_FillValue = -999  ;Fill with missing values

end if


if(scenarioA.eq."FUT_CLIM") then

fAin = fC_dir + "EPIC_"+var+".sum_2045_2054_ave_"+year_setA + ".nc"
fA=addfile(fAin,"r")


;This is subsetted data for selected states
print("Subsetting data for selected states...")

xA = fA->L2_ANH3
xA = -999
xB = fA->L2_ANH3
xA@_FillValue = -999  ;Fill with missing values

end if



;---------updated, much faster loop
print("Loop to put in state specific values...")
;Now Loop to put in state specific values
do a = 0, dimsizes(lat1d_new3) - 1
xA(:,:,ROW_ind(a),COL_ind(a))=xB(:,:,ROW_ind(a),COL_ind(a))  ;Even Much faster loop
end do
;-------------------------------
xA_sum=dim_sum_n_Wrap(xA,1)  ;sums over all crop types
xA_avg=avg(xA_sum)           ;area average
data_plots(i,j) = xA_avg

delete(xA)
delete(xB)
end do
end do


FUT_CURR_diff = avg(data_plots(1,:)) - avg(data_plots(0,:))
print("Avg. FUT - CURR (K) = " + FUT_CURR_diff)
FUT_CURR_months = (data_plots(1,:)) - (data_plots(0,:))
print("Months FUT - CURR (K) = " + FUT_CURR_months)



monthA=(/"J","F","M","A","M","J","J","A","S","O","N","D"/)
x = ispan (1,12,1)
;------------------------------------------------
; Plot the timeseries
;------------------------------------------------
  nVar    = 1
  fo = "plots/L2_ANH3.CURR_FUT.Monthly.CHESBAY_air"
  wks  = gsn_open_wks ("eps",fo)


  res  = True
  res@gsnDraw                = True
  res@gsnFrame               = True
  res@xyLineThicknessF       = 2
  res@tiYAxisString          = "Month"
;  res@tiYAxisString          = "PDF (%)"
  res@tiYAxisString          = "L2_ANH3"
  res@trYMaxF           = 50.                    ; axis max
  res@trYMinF           = 0.                    ; axis min
  res@trXMaxF           = 12                    ; axis max
  res@trXMinF           = 1                    ; axis min


; res@xyDashPattern  = 0                         ; Make curves all solid
 res@xyMarkLineMode = "MarkLines"               ; Markers *and* lines
 res@xyMarkers      = (/16,16,6/)               ; 3 different marker
 res@xyLineThicknesses = (/  5.0,   5.0, 5.0,   2.0/)          ; make second line thicker
  res@xyLineColors      = (/"blue","red","green","mediumblue"/)                    ; line colors
 res@xyMarkerColors := (/"blue","red","green"/) ; 3 different colors
   res@gsnCenterString        = " Average Monthly L2_ANH3"

;  res@xyDashPattern  = 0                         ; Make curves all solid

   res@pmLegendDisplayMode    = "Always"            ; turn on legend
  res@pmLegendSide           = "Top"               ; Change location of 
  res@pmLegendParallelPosF   = .20                  ; move units right
  res@pmLegendOrthogonalPosF = -0.3                ; move units down
  res@pmLegendWidthF         = 0.10                ; Change width and
  res@pmLegendHeightF        = 0.15                ; height of legend.
  res@lgPerimOn              = False               ; turn off box around
  res@lgLabelFontHeightF     = .01                 ; label font height
  res@xyExplicitLegendLabels = (/"EPIC CURR (1995-2004)","EPIC FUT_CLIM (2045-2054)"/)         ; create explicit labels




  plot = gsn_csm_xy (wks,x,data_plots, res) 

delete(wks)
 system("convert   -trim  -page A4+0+0 "+fo+".eps "+fo+".png")


end
